# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String

logger = logging.getLogger("driverng.cfx_neoplex")


class CfxNeoplexBase(Splitted):
    SEPARATOR = ","


class CfxNeoplexResult(object):
    FIELDS = [
        String("no"),
        String("name"),
        String("well"),
        String("sample_type"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class CfxNeoplexResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: CfxNeoplexResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, skipinitialspace=True)
        self.analysis_date = None
        for index, line in enumerate(list(reader)):
            if line[0] == "Analysis Date:":
                try:
                    self.analysis_date = datetime.datetime.strptime(
                        line[1].strip(), "%Y-%m-%d %H:%M:%S"
                    )
                except ValueError as e:
                    logger.info("La date des résultats n'est pas au bon format %s", e)
            # remove the 5 first lines
            if index in [0, 1, 2, 3, 4]:
                continue
            obj = self.add(type_message=MessageV2.RESULT)
            obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "cfx_neoplex", "label": "CFX Logiciel NeoPlex"}

    def parse(self, data):
        obj = CfxNeoplexResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        detected = False
        RESULTS = ()

        for m in message:
            sample_id = rack = ""
            if m.name and m.name != "Name":
                sample_id = m.name
                rack = m.well
            if m.name == "Name":
                NAMES = tuple(m.results)
            if sample_id and m.sample_type != "SAMPLE":
                for result in m.results:
                    if "Not detected" not in result:
                        detected = True
                        control_id = "{}_{}".format(sample_id, m.no)
                        sample = worklist.get_or_create(
                            sample_id=control_id,
                            control_lot=control_id,
                            rack=rack,
                            type="control",
                        )
                        CODES = tuple(m.results)
            elif sample_id and sample_id != "/" and m.sample_type == "SAMPLE":
                for result in m.results:
                    if "Not detected" not in result:
                        detected = True
                        sample = worklist.get_or_create(
                            sample_id=sample_id, rack=rack, type="patient"
                        )
                        CODES = tuple(m.results)
            elif not sample_id and detected:
                RESULTS = tuple(m.results)
                for i in range(len(CODES)):
                    if CODES[i] and RESULTS[i]:
                        sample.results.add(
                            code=CODES[i], value=RESULTS[i], date=message.analysis_date
                        )
                    # for IC, Result and Comment
                    if CODES[i] and not RESULTS[i]:
                        sample.results.add(
                            code=NAMES[i], value=CODES[i], date=message.analysis_date
                        )
                detected = False

        return worklist
